// http://wwwinfo.cern.ch/asd/lhc++/clhep/manual/RefGuide/random.html

#include <vector>
using std::vector;

#include "rng.h"
#include "CLHEP/Random/RanluxEngine.h"

class CLHEPRandomNumberGenerator : public RandomNumberGenerator
{
public:
  CLHEPRandomNumberGenerator(HepRandomEngine *const hre, const long int seed, const int luxury);
  ~CLHEPRandomNumberGenerator();
  
  const double uniformInZeroOne();
  void uniformInZeroOne(vector<double>& vectorOfRandomNumbers);
  void setSeedAndLuxuryLevel(const long int seed, const int luxury);
  void varySeedToGetIndependentRandomNumberStreams(const long int unique);
private:
  long int _seed;
  int _luxury;
  HepRandomEngine *const _clhepRandomNumberGenerator;
private:     // prevent copy construction and assignment
  CLHEPRandomNumberGenerator(const CLHEPRandomNumberGenerator&);
  CLHEPRandomNumberGenerator& operator=(const CLHEPRandomNumberGenerator&);  
};

// --------------------------------------------------------------

RandomNumberGenerator *const randomNumberGenerator = new CLHEPRandomNumberGenerator(new RanluxEngine(), 19780503, 3);

// --------------------------------------------------------------

CLHEPRandomNumberGenerator::CLHEPRandomNumberGenerator(HepRandomEngine *const hre, const long int seed, const int luxury)
  : _seed(0), _luxury(0), _clhepRandomNumberGenerator(hre)
{
  setSeedAndLuxuryLevel(seed, luxury);
}

CLHEPRandomNumberGenerator::~CLHEPRandomNumberGenerator()
{
  delete _clhepRandomNumberGenerator;
}

const double CLHEPRandomNumberGenerator::uniformInZeroOne()
{
  return _clhepRandomNumberGenerator->flat();
}

void CLHEPRandomNumberGenerator::uniformInZeroOne(vector<double>& vectorOfRandomNumbers)
{
  _clhepRandomNumberGenerator->flatArray(vectorOfRandomNumbers.size(), &vectorOfRandomNumbers[0]);
}

void CLHEPRandomNumberGenerator::setSeedAndLuxuryLevel(const long int seed, const int luxury)
{
  _seed = seed;
  _luxury = luxury;
  _clhepRandomNumberGenerator->setSeed(seed, luxury);
}

void CLHEPRandomNumberGenerator::varySeedToGetIndependentRandomNumberStreams(const long int unique)
{
  setSeedAndLuxuryLevel(_seed + unique, _luxury);
}
